@@ -1,4 +1,3 @@
-import importlib
 import os
 import time
 import json
@@ -18,6 +17,10 @@ from .autofrontend import register_autofrontend
 from .base import MLServer, Converter, RawResponse, FileResponse, TemplateResponse
 from .format import RawFormat
 from .view import View
+from mlchain import mlchain_context
+from sentry_sdk import push_scope, start_transaction
+from mlchain import mlconfig
+from uuid import uuid4
 
 APP_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 TEMPLATE_PATH = os.path.join(APP_PATH, 'server/templates')
@@ -69,6 +72,11 @@ class FlaskEndpointAction:
         return Response(output_encoded, mimetype='application/msgpack_blosc',
                         status=status)
 
+    def init_context(self): 
+        uid = str(uuid4())
+        mlchain_context['MLCHAIN_CONTEXT_ID'] = uid
+        return uid 
+
     def __call__(self, *args, **kwargs):
         """
         Standard method that effectively perform the stored action of this endpoint.
@@ -78,104 +86,115 @@ class FlaskEndpointAction:
         """
         start_time = time.time()
 
-        # If data POST is in msgpack format
-        serializer = self.serializers_dict.get(
-            request.content_type,
-            self.serializers_dict[request.headers.get('serializer', 'application/json')]
-        )
-        if request.content_type == 'application/msgpack':
-            response_function = self.__get_msgpack_response
-        elif request.content_type == 'application/msgpack_blosc':
-            response_function = self.__get_msgpack_blosc_response
-        else:
-            response_function = self.__get_json_response
-        if request.method == 'POST' and self.api_keys is not None or (
-                isinstance(self.api_keys, (list, dict)) and len(self.api_keys) > 0):
-            authorized = False
-            has_key = False
-            for key in ['x-api-key', 'apikey', 'apiKey', 'api-key']:
-                apikey = request.headers.get(key, '')
-                if apikey != '':
-                    has_key = True
-                if apikey in self.api_keys:
-                    authorized = True
-                    break
-            if not authorized:
-                if has_key:
-                    error = 'Unauthorized. Api-key incorrect.'
+        with push_scope() as sentry_scope:
+            transaction_name = "{0}  ||  {1}".format(mlconfig.MLCHAIN_SERVER_NAME, self.action.__name__)
+            sentry_scope.transaction = transaction_name
+            
+            with start_transaction(op="task", name=transaction_name):
+                uid = self.init_context()
+
+                # If data POST is in msgpack format
+                serializer = self.serializers_dict.get(
+                    request.content_type,
+                    self.serializers_dict[request.headers.get('serializer', 'application/json')]
+                )
+                if request.content_type == 'application/msgpack':
+                    response_function = self.__get_msgpack_response
+                elif request.content_type == 'application/msgpack_blosc':
+                    response_function = self.__get_msgpack_blosc_response
                 else:
-                    error = 'Unauthorized. Lack of x-api-key or apikey or api-key in headers.'
-                output = {
-                    'error': error,
-                    'api_version': self.version,
-                    'mlchain_version': mlchain.__version__
-                }
-                return response_function(output, 401)
-        try:
-            # Perform the action
-            if request.method == 'POST':
-                output = self.action(*args, **kwargs, serializer=serializer)
-            else:
-                output = self.action(*args, **kwargs)
-            if isinstance(output, RawResponse):
-                output = Response(output.response, status=output.status,
-                                  headers=output.headers,
-                                  mimetype=output.mimetype,
-                                  content_type=output.content_type)
-                output.headers['mlchain_version'] = mlchain.__version__
-                output.headers['api_version'] = self.version
-                return output
-            if isinstance(output, FileResponse):
-                file = send_file(output.path, mimetype=output.mimetype)
-                for k, v in output.headers.items():
-                    file.headers[k] = v
-                file.headers['mlchain_version'] = mlchain.__version__
-                file.headers['api_version'] = self.version
-                return file
-            if isinstance(output, Response):
-                return output
+                    response_function = self.__get_json_response
+                if request.method == 'POST' and self.api_keys is not None or (
+                        isinstance(self.api_keys, (list, dict)) and len(self.api_keys) > 0):
+                    authorized = False
+                    has_key = False
+                    for key in ['x-api-key', 'apikey', 'apiKey', 'api-key']:
+                        apikey = request.headers.get(key, '')
+                        if apikey != '':
+                            has_key = True
+                        if apikey in self.api_keys:
+                            authorized = True
+                            break
+                    if not authorized:
+                        if has_key:
+                            error = 'Unauthorized. Api-key incorrect.'
+                        else:
+                            error = 'Unauthorized. Lack of x-api-key or apikey or api-key in headers.'
+                        output = {
+                            'error': error,
+                            'api_version': self.version,
+                            'mlchain_version': mlchain.__version__,
+                            "request_id": mlchain_context.MLCHAIN_CONTEXT_ID
+                        }
+                        return response_function(output, 401)
+                try:
+                    # Perform the action
+                    if request.method == 'POST':
+                        output = self.action(*args, **kwargs, serializer=serializer)
+                    else:
+                        output = self.action(*args, **kwargs)
+                    if isinstance(output, RawResponse):
+                        output = Response(output.response, status=output.status,
+                                        headers=output.headers,
+                                        mimetype=output.mimetype,
+                                        content_type=output.content_type)
+                        output.headers['mlchain_version'] = mlchain.__version__
+                        output.headers['api_version'] = self.version
+                        output.headers['request_id'] = mlchain_context.MLCHAIN_CONTEXT_ID
+                        return output
+                    if isinstance(output, FileResponse):
+                        file = send_file(output.path, mimetype=output.mimetype)
+                        for k, v in output.headers.items():
+                            file.headers[k] = v
+                        file.headers['mlchain_version'] = mlchain.__version__
+                        file.headers['api_version'] = self.version
+                        file.headers['request_id'] = mlchain_context.MLCHAIN_CONTEXT_ID
+                        return file
+                    if isinstance(output, Response):
+                        return output
 
-            output = {
-                'output': output,
-                'time': round(time.time() - start_time, 2),
-                'api_version': self.version,
-                'mlchain_version': mlchain.__version__
-            }
-            return response_function(output, 200)
-        except MlChainError as ex:
-            err = ex.msg
-            logger.error("code: {0} msg: {1}".format(ex.code, ex.msg))
+                    output = {
+                        'output': output,
+                        'time': round(time.time() - start_time, 2),
+                        'api_version': self.version,
+                        'mlchain_version': mlchain.__version__,
+                        "request_id": mlchain_context.MLCHAIN_CONTEXT_ID
+                    }
+                    return response_function(output, 200)
+                except MlChainError as ex:
+                    err = ex.msg
 
-            output = {
-                'error': err,
-                'time': round(time.time() - start_time, 2),
-                'code': ex.code,
-                'api_version': self.version,
-                'mlchain_version': mlchain.__version__
-            }
-            return response_function(output, ex.status_code)
-        except AssertionError as ex:
-            err = str(ex)
-            logger.error(err)
+                    output = {
+                        'error': err,
+                        'time': round(time.time() - start_time, 2),
+                        'code': ex.code,
+                        'api_version': self.version,
+                        'mlchain_version': mlchain.__version__,
+                        "request_id": mlchain_context.MLCHAIN_CONTEXT_ID
+                    }
+                    return response_function(output, ex.status_code)
+                except AssertionError as ex:
+                    err = str(ex)
 
-            output = {
-                'error': err,
-                'time': round(time.time() - start_time, 2),
-                'api_version': self.version,
-                'mlchain_version': mlchain.__version__
-            }
-            return response_function(output, 422)
-        except Exception:
-            err = str(format_exc(name='mlchain.serve.server'))
-            logger.error(err)
+                    output = {
+                        'error': err,
+                        'time': round(time.time() - start_time, 2),
+                        'api_version': self.version,
+                        'mlchain_version': mlchain.__version__,
+                        "request_id": mlchain_context.MLCHAIN_CONTEXT_ID
+                    }
+                    return response_function(output, 422)
+                except Exception:
+                    err = format_exc(name='mlchain.serve.server', return_str=False)
 
-            output = {
-                'error': err,
-                'time': round(time.time() - start_time, 2),
-                'api_version': self.version,
-                'mlchain_version': mlchain.__version__
-            }
-            return response_function(output, 500)
+                    output = {
+                        'error': err,
+                        'time': round(time.time() - start_time, 2),
+                        'api_version': self.version,
+                        'mlchain_version': mlchain.__version__,
+                        "request_id": mlchain_context.MLCHAIN_CONTEXT_ID
+                    }
+                    return response_function(output, 500)
 
 
 class FlaskView(View):
@@ -192,8 +211,8 @@ class FlaskView(View):
             return headers, form, files, data
         except RequestEntityTooLarge:
             raise MlChainError("Request too large", status_code=413)
-        except:
-            raise
+        except Exception as ex:
+            raise MlChainError("Parse request data error: {0}".format(ex), status_code=500)
 
     def make_response(self, response: Union[RawResponse, FileResponse]):
         if isinstance(response, RawResponse):
@@ -203,6 +222,7 @@ class FlaskView(View):
                               content_type=response.content_type)
             output.headers['mlchain_version'] = mlchain.__version__
             output.headers['api_version'] = self.server.version
+            output.headers['request_id'] = mlchain_context.MLCHAIN_CONTEXT_ID
             return output
             
         if isinstance(response, FileResponse):
@@ -211,6 +231,7 @@ class FlaskView(View):
                 file.headers[k] = v
             file.headers['mlchain_version'] = mlchain.__version__
             file.headers['api_version'] = self.server.version
+            file.headers['request_id'] = mlchain_context.MLCHAIN_CONTEXT_ID
             return file
 
         if isinstance(response, TemplateResponse):
@@ -224,37 +245,29 @@ class FlaskView(View):
 class FlaskServer(MLServer):
     def __init__(self, model: ServeModel, name=None, version='0.0',
                  authentication=None, api_format=None,
-                 static_folder=None, template_folder=None, static_url_path=None):
-        MLServer.__init__(self, model, name)
+                 static_folder=None, template_folder=None, static_url_path:str="static"):
+        MLServer.__init__(self, model, name, version, api_format, authentication)
+
+        if not isinstance(static_url_path, str): 
+            static_url_path = "static"
         self.app = Flask(self.name, static_folder=static_folder,
                          template_folder=template_folder,
                          static_url_path=static_url_path)
-        self.app.config['MAX_CONTENT_LENGTH'] = 200 * 1024 * 1024
+
+        self.app.url_map.strict_slashes = False
+        self.app.config['MAX_CONTENT_LENGTH'] = 1024 * 1024 * 1024
         self.app.config['JSONIFY_PRETTYPRINT_REGULAR'] = True
 
-        self.version = version
         self.converter = Converter(FileStorage, self._get_file_name, self._get_data)
-        self.register_home()
-        self.authentication = authentication
-        self._initalize_app()
-        if isinstance(api_format, str):
-            try:
-                package, class_name = api_format.rsplit('.', 1)
-                api_format = importlib.import_module(package)
-                api_format = getattr(api_format, class_name)
-            except:
-                api_format = None
-        if isinstance(api_format, type):
-            api_format = api_format()
 
-        self.api_format = '{0}.{1}'.format(api_format.__class__.__module__,
-                                           api_format.__class__.__name__)
         self.app.add_url_rule('/call/<function_name>', 'call',
-                              FlaskView(self, api_format, self.authentication),
+                              FlaskView(self, self.api_format_class, self.authentication),
                               methods=['POST', 'GET'], strict_slashes=False)
         self.app.add_url_rule('/call_raw/<function_name>', 'call_raw',
                               FlaskView(self, RawFormat(), self.authentication),
                               methods=['POST', 'GET'], strict_slashes=False)
+        
+        self.initialize_endpoint()
 
     def _get_file_name(self, storage):
         return storage.filename
@@ -278,7 +291,7 @@ class FlaskServer(MLServer):
                                                   api_keys=None),
                               methods=methods)
 
-    def register_swagger(self):
+    def _register_swagger(self):
         swagger_ui = Blueprint("swagger", __name__,
                                static_folder=os.path.join(TEMPLATE_PATH, 'swaggerui'))
 
@@ -288,7 +301,15 @@ class FlaskServer(MLServer):
                                            description=self.model.model.__doc__,
                                            version=self.model.name)
         for name, func in self.model.get_all_func().items():
-            swagger_template.add_endpoint(func, f'/call/{name}', tags=[self.name])
+            swagger_template.add_endpoint(func, f'/call/{name}', tags=["MlChain Format APIs"])
+            swagger_template.add_endpoint(func, f'/call_raw/{name}', tags=["MlChain Raw Output APIs"])
+
+        swagger_template.add_core_endpoint(self.model._get_parameters_of_func, '/api/get_params/{function_name}', tags=["MlChain Core APIs"])
+        swagger_template.add_core_endpoint(self.model._get_description_of_func, '/api/des_func/{function_name}', tags=["MlChain Core APIs"])
+        swagger_template.add_core_endpoint(self._check_status, '/api/ping', tags=["MlChain Core APIs"])
+        swagger_template.add_core_endpoint(self.model._get_all_description, '/api/description', tags=["MlChain Core APIs"])
+        swagger_template.add_core_endpoint(self.model._list_all_function, '/api/list_all_function', tags=["MlChain Core APIs"])
+        swagger_template.add_core_endpoint(self.model._list_all_function_and_description, '/api/list_all_function_and_description', tags=["MlChain Core APIs"])
 
         SWAGGER_URL = '/swagger'
 
@@ -311,7 +332,7 @@ class FlaskServer(MLServer):
 
         self.app.register_blueprint(swagger_ui)
 
-    def register_home(self):
+    def _register_home(self):
         home_ui = Blueprint("home",
                             __name__,
                             static_folder=STATIC_PATH,
@@ -325,9 +346,9 @@ class FlaskServer(MLServer):
         self.app.register_blueprint(home_ui)
 
     def run(self, host='127.0.0.1', port=8080, bind=None, cors=False, cors_resources={},
-            cors_allow_origins='*', gunicorn=False, debug=False,
-            use_reloader=False, workers=1, timeout=60, keepalive=10,
-            max_requests=0, threads=1, worker_class='gthread', umask='0',
+            cors_allow_origins:list=['*'], gunicorn=False, debug=False,
+            use_reloader=False, workers=1, timeout=200, keepalive=3,
+            max_requests=0, threads=1, worker_class='gthread', 
             ngrok=False, model_id=None, **kwargs):
         """
         Run a server from a Python class
@@ -340,7 +361,7 @@ class FlaskServer(MLServer):
         :whitelist: Served all function name inside whitelist
         :cors: Enable CORS or not
         :cors_resources: Config Resources of flask-cors
-        :cors_allow_origins: Allow host of cors
+        :cors_allow_origins: Allow hosts of cors
         :gunicorn: Run with Gunicorn or not
         :debug: Debug or not
         :use_reloader: Default False, which is using 1 worker in debug instead of 2
@@ -350,7 +371,6 @@ class FlaskServer(MLServer):
         :threads: The number of worker threads for handling requests. Be careful, threads would break your result if it is bigger than 1
         :worker_class: The type of workers to use.
         :max_requests: Max Request to restart Gunicorn Server, default is 0 which means no restart
-        :umask: A bit mask for the file mode on files written by Gunicorn.
         :kwargs: Other Gunicorn options
         """
         try:
@@ -375,6 +395,7 @@ class FlaskServer(MLServer):
 
         if cors:
             CORS(self.app, resources=cors_resources, origins=cors_allow_origins)
+        
         if not gunicorn:
             if bind is not None:
                 if isinstance(bind, str):
@@ -401,6 +422,10 @@ class FlaskServer(MLServer):
             if bind is None:
                 bind = [bind_host_port]
 
+            if "uvicorn" in worker_class.lower(): 
+                logger.warning("Change worker_class from {0} to {1} due to compatible with Flask".format(worker_class, "gthread"))
+                worker_class = "gthread"
+
             logger.info("-" * 80)
             logger.info("Served model with Flask and Gunicorn at bind={}".format(bind))
             logger.info("Number of workers: {}".format(workers))
@@ -410,7 +435,8 @@ class FlaskServer(MLServer):
             logger.info("-" * 80)
 
             loglevel = kwargs.get('loglevel', 'warning' if debug else 'info')
+            
             GunicornWrapper(self.app, bind=bind, workers=workers, timeout=timeout,
                             keepalive=keepalive, max_requests=max_requests,
                             loglevel=loglevel, worker_class=worker_class,
-                            threads=threads, umask=umask, **kwargs).run()
+                            threads=threads, **kwargs).run()
\ No newline at end of file
